@@ -36,8 +36,8 @@ Nthin <- Nprt <- 10
 Nbi <- Nprb <- 300
 
 est <- ebsglmm(Infected ~ 1, 'binomial', rhizdata, weights = Total,
-               atsample = ~ Xcoord + Ycoord, parameters = parlist,
-               estimate = estimate, corrfcn = corrf, 
+               atsample = ~ Xcoord + Ycoord, parskel = parlist,
+               paroptim = estimate, corrfcn = corrf, 
                Nout = Nout, Nthin = Nthin, Nbi = Nbi,
                Npro = Npro, Nprt = Nprt, Nprb = Nprb, 
                betm0 = betm0, betQ0 = betQ0, ssqdf = ssqdf, ssqsc = ssqsc,
